<?php

/**
 * @file
 * Drush commands for content_synchronizer module.
 */

use Drupal\content_synchronizer\Entity\ImportEntity;

/**
 * Implementation of hook_drush_command().
 */
function content_synchronizer_drush_command() {
  $items = [];

  $items['content-synchronizer-create-import'] = [
    'description'        => 'Create an import from passed .zip file.',
    'arguments'          => [
      'file' => 'Optional. The cache bin to fetch from.',
    ],
    'required-arguments' => 1,
    'aliases'            => ['csci'],
  ];

  return $items;
}

/**
 * Create an import entity from a zip file.
 *
 * @param string $path
 *   The path to the file to import.
 */
function drush_content_synchronizer_create_import($path) {
  dump($path);
  if (file_exists($path)) {
    $extensionData = explode('.', $path);
    if (end($extensionData) == 'zip') {
      if ($file = file_save_data(file_get_contents($path))) {
        $name = strip_tags(t('Drush import - %date', ['%date' => \Drupal::service('date.formatter')->format(time())]));
        $ie = ImportEntity::create(
          [
            'name' => $name,
            ImportEntity::FIELD_ARCHIVE => $file
          ]
        );
        $ie->save();
      }
    }
    else {
      drush_log('The file is not a .zip archive', 'error');
    }
  }
  else {
    drush_log('No file found', 'error');
  }

}
