<?php

namespace Drupal\content_synchronizer\Form;

use Drupal\content_synchronizer\Entity\ImportEntity;
use Drupal\content_synchronizer\Processors\BatchImportProcessor;
use Drupal\content_synchronizer\Processors\ImportProcessor;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Launch Import Form.
 */
class LaunchImportForm extends FormBase {

  /**
   * The import entity.
   *
   * @var \Drupal\content_synchronizer\Entity\ImportEntity
   */
  protected $import;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_synchronizer.import.launch';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    /** @var \Drupal\content_synchronizer\Entity\ImportEntity $import */
    $this->import = $form_state->getBuildInfo()['import'];

    if ($this->import->getProcessingStatus() === ImportEntity::STATUS_NOT_STARTED) {
      // Settings.
      $form['settings'] = [
        '#type'  => 'fieldset',
        '#title' => t('Settings'),
      ];

      $form['settings']['creationType'] = [
        '#type'          => 'radios',
        '#title'         => t('Action on entity creation'),
        '#options'       => [
          ImportProcessor::PUBLICATION_PUBLISH   => t('Publish created content'),
          ImportProcessor::PUBLICATION_UNPUBLISH => t('Do not publish created content'),
        ],
        '#default_value' => ImportProcessor::PUBLICATION_PUBLISH
      ];

      $form['settings']['updateType'] = [
        '#type'          => 'radios',
        '#title'         => t('Action on entity update'),
        '#options'       => [
          ImportProcessor::UPDATE_SYSTEMATIC => t('Always update existing entity with importing content'),
          ImportProcessor::UPDATE_IF_RECENT  => t('Update existing entity with importing content only if the last change date of importing content is more recent than the last change date of the corresponding existing entity'),
          ImportProcessor::UPDATE_NO_UPDATE  => t('Do not update existing content'),
        ],
        '#default_value' => ImportProcessor::UPDATE_IF_RECENT
      ];
    }

    // Entity list.
    $this->initRootEntitiesList($form);
    if ($this->import->getProcessingStatus() === ImportEntity::STATUS_NOT_STARTED) {

      $form['launch'] = [
        '#type'        => 'submit',
        '#value'       => t('Import selected entities'),
        '#button_type' => 'primary',
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $batchImport = new BatchImportProcessor();
    $batchImport->setEntitiesToImport(array_intersect_key($this->import->getRootsEntities(), array_flip($form_state->getUserInput()['entities_to_import'])));

    $batchImport->import($this->import, [
      $this,
      'onBatchEnd'
    ], $form_state->getValue('creationType'), $form_state->getValue('updateType'));
  }

  /**
   * The callback after batch process.
   */
  public function onBatchEnd($data) {
    $this->import->removeArchive();
  }

  /**
   * Init the root entities list for display.
   */
  protected function initRootEntitiesList(array &$form) {
    $rootEntities = $this->import->getRootsEntities();
    $build = [
      '#theme'         => 'entities_list_table',
      '#entities'      => $rootEntities,
      '#checkbox_name' => 'entities_to_import[]',
      '#title'         => t('Entities to import'),
    ];

    $form['entities_list'] = $build;
  }

}
